@@ -111,7 +111,7 @@ couchdb_session_set_property (GObject         *object,
 	switch (prop_id) {
 	case PROP_URI:
 		g_free(couchdb->priv->uri);
-		couchdb->priv->uri = g_value_dup_string(value);
+		couchdb->priv->uri = g_value_dup_string (value);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -248,6 +248,9 @@ couchdb_session_init (CouchdbSession *couchdb)
 CouchdbSession *
 couchdb_session_new (const char *uri)
 {
+	if (!uri)
+		uri = "http://127.0.0.1:5984";
+
 	return g_object_new (COUCHDB_TYPE_SESSION, "uri", uri, NULL);
 }
 
@@ -495,9 +498,27 @@ couchdb_session_free_database_list (GSList *dblist)
 	g_slist_free (dblist);
 }
 
+static JsonArray *
+call_all_docs_uri (CouchdbSession *session, const char *url, JsonParser *parser, GError **error)
+{
+	JsonArray *rows = NULL;
+
+	if (couchdb_session_send_message (session, SOUP_METHOD_GET, url, NULL, parser, error)) {
+		JsonNode *root_node;
+
+		root_node = json_parser_get_root (parser);
+		if (json_node_get_node_type (root_node) == JSON_NODE_OBJECT) {
+			rows = json_object_get_array_member (
+				json_node_get_object (root_node), "rows");
+		}
+	}
+
+	return rows;
+}
+
 /**
  * couchdb_session_list_documents:
- * @couchdb: A #CouchdbSession object
+ * @session: A #CouchdbSession object
  * @dbname: Name of the databases to retrieve documents from
  * @error: Placeholder for error information
  *
@@ -511,42 +532,91 @@ couchdb_session_free_database_list (GSList *dblist)
  * #couchdb_session_free_document_list.
  */
 GSList *
-couchdb_session_list_documents (CouchdbSession *couchdb, const char *dbname, GError **error)
+couchdb_session_list_documents (CouchdbSession *session, const char *dbname, GError **error)
 {
 	char *url;
 	JsonParser *parser;
+	JsonArray *rows;
 	GSList *doclist = NULL;
 
-	g_return_val_if_fail (COUCHDB_IS_SESSION (couchdb), NULL);
+	g_return_val_if_fail (COUCHDB_IS_SESSION (session), NULL);
 	g_return_val_if_fail (dbname != NULL, NULL);
 
-	url = g_strdup_printf ("%s/%s/_all_docs", couchdb->priv->uri, dbname);
+	url = g_strdup_printf ("%s/%s/_all_docs", session->priv->uri, dbname);
 	parser = json_parser_new ();
-	if (couchdb_session_send_message (couchdb, SOUP_METHOD_GET, url, NULL, parser, error)) {
-		JsonNode *root_node;
 
-		root_node = json_parser_get_root (parser);
-		if (json_node_get_node_type (root_node) == JSON_NODE_OBJECT) {
-			JsonArray *rows;
-			gint i;
+	rows = call_all_docs_uri (session, url, parser, error);
+	if (rows != NULL) {
+		gint i;
+		for (i = 0; i < json_array_get_length (rows); i++) {
+			JsonObject *doc;
+			CouchdbDocumentInfo *doc_info;
 
-			rows = json_object_get_array_member (
-				json_node_get_object (root_node), "rows");
-			for (i = 0; i < json_array_get_length (rows); i++) {
-				JsonObject *doc;
-				CouchdbDocumentInfo *doc_info;
+			doc = json_array_get_object_element (rows, i);
+			if (!doc)
+				continue;
 
-				doc = json_array_get_object_element (rows, i);
-				if (!doc)
-					continue;
+			doc_info = couchdb_document_info_new (
+				json_object_get_string_member (doc, "id"),
+				json_object_get_string_member (
+					json_object_get_object_member (doc, "value"),
+					"rev"));
+			doclist = g_slist_append (doclist, doc_info);
+		}
+	}
 
-				doc_info = couchdb_document_info_new (
-					json_object_get_string_member (doc, "id"),
-					json_object_get_string_member (
-						json_object_get_object_member (doc, "value"),
-						"rev"));
-				doclist = g_slist_append (doclist, doc_info);
-			}
+	g_object_unref (G_OBJECT (parser));
+	g_free (url);
+
+	return doclist;
+}
+
+/**
+ * couchdb_session_get_all_documents:
+ * @couchdb: A #CouchdbSession object
+ * @dbname: Name of the databases to retrieve documents from
+ * @error: Placeholder for error information
+ *
+ * Retrieve all documents from a database on a running CouchDB instance. For
+ * each document found in the database, a #CouchdbDocument object is returned
+ * on the list, which represents the document's contents as found on the
+ * underlying database.
+ *
+ * Return value: a list of #CouchdbDocument objects, or NULL if there are none
+ * or there was an error (in which case the error argument will contain information
+ * about the error). Once no longer needed, the list should be freed by calling
+ * #couchdb_session_free_document_list.
+ */
+GSList *
+couchdb_session_get_all_documents (CouchdbSession *session, const char *dbname, GError **error)
+{
+	char *url;
+	JsonParser *parser;
+	JsonArray *rows;
+	GSList *doclist = NULL;
+
+	g_return_val_if_fail (COUCHDB_IS_SESSION (session), NULL);
+	g_return_val_if_fail (dbname != NULL, NULL);
+
+	url = g_strdup_printf ("%s/%s/_all_docs?include_docs=true", session->priv->uri, dbname);
+	parser = json_parser_new ();
+
+	rows = call_all_docs_uri (session, url, parser, error);
+	if (rows != NULL) {
+		gint i;
+		for (i = 0; i < json_array_get_length (rows); i++) {
+			JsonObject *obj;
+			CouchdbDocument *document;
+
+			obj = json_array_get_object_element (rows, i);
+			if (!obj)
+				continue;
+
+			document = couchdb_document_new_from_json_object (
+				session,
+				json_object_get_object_member (obj, "doc"));
+			if (document != NULL)
+				doclist = g_slist_append (doclist, document);
 		}
 	}
 
@@ -568,8 +638,16 @@ couchdb_session_free_document_list (GSList *doclist)
 {
 	g_return_if_fail (doclist != NULL);
 
-	g_slist_foreach (doclist, (GFunc) couchdb_document_info_unref, NULL);
-	g_slist_free (doclist);
+	while (doclist != NULL) {
+		gpointer data = doclist->data;
+
+		doclist = g_slist_remove (doclist, data);
+
+		if (COUCHDB_IS_DOCUMENT (data))
+			g_object_unref (G_OBJECT (data));
+		else
+			couchdb_document_info_unref (data);
+	}
 }
 
 /**
