@@ -115,6 +115,10 @@ test_change_databases (void)
 	dbname = generate_uuid ();
 	g_assert (dbname != NULL);
 
+	/* Database name can not start with a digit
+	 * we will overwrite the first character with 'a' */
+	 dbname[0] = 'a';
+
 	/* Create database */
 	couchdb_session_create_database (couchdb, dbname, &error);
 	if (error) {
@@ -192,7 +196,7 @@ main (int argc, char *argv[])
 	g_test_init (&argc, &argv, NULL);
 
 	/* Initialize data needed for all tests */
-	couchdb =  argc > 1 ? couchdb_session_new (argv[1]) : couchdb_session_new (NULL);
+	couchdb =  argc > 1 ? couchdb_session_new (argv[1]) : couchdb_session_new ("http://test:test@127.0.0.1:5985");
 	g_printf ("Connecting to Couchdb at %s\n", couchdb_session_get_uri (couchdb));
 	
 	if (!couchdb) {
